/* gAlan - Graphical Audio Language
 * Copyright (C) 1999 Tony Garnock-Jones
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stddef.h>

#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gmodule.h>

#include "global.h"
#include "generator.h"
#include "comp.h"
#include "control.h"
#include "gencomp.h"

#define LOCATION(g)	((SAMPLETIME) ((g)->data))

PRIVATE gboolean init_instance(Generator *g) {
  LOCATION(g) = -1;
  return TRUE;
}

PRIVATE gboolean output_generator(Generator *g, SAMPLE *buf, int buflen) {
  SAMPLETIME loc = LOCATION(g);

  if (loc < 0)
    return FALSE;

  LOCATION(g) += buflen;
  return gen_read_randomaccess_input(g, 0, 0, loc, buf, buflen);
}

PRIVATE void evt_reset_handler(Generator *g, AEvent *event) {
  LOCATION(g) = 0;
}

PRIVATE InputSignalDescriptor input_sigs[] = {
  { "Input", SIG_FLAG_RANDOMACCESS },
  { NULL, }
};

PRIVATE OutputSignalDescriptor output_sigs[] = {
  { "Output", SIG_FLAG_REALTIME, { output_generator, } },
  { NULL, }
};

PRIVATE ControlDescriptor controls[] = {
  /* { kind, name, min,max,step,page, size,editable, is_dst,queue_number,
       init,destroy,refresh,refresh_data }, */
  { CONTROL_KIND_NONE, }
};

PRIVATE void setup_class(void) {
  GeneratorClass *k = gen_new_generatorclass("rart", 1, 0,
					     input_sigs, output_sigs, controls,
					     init_instance, NULL,
					     (AGenerator_pickle_t) init_instance, NULL);

  gen_configure_event_input(k, 0, "Reset", evt_reset_handler);

  gencomp_register_generatorclass(k, "Misc/Randomaccess to Realtime Converter", NULL, NULL);
}

PUBLIC void init_plugin_rart(void) {
  setup_class();
}
